#version 330 core

layout(location=0) out vec4 fragColor;

in vec3 P;
in vec3 N;
in vec2 T;
in vec4 S;

in vec3 MainLightDirection_tangentspace;
in vec3 EyeDirection_tangentspace;
in vec3 TriangleNormalTangentSpace;
in vec3 pLightDirection_tangentspace[12];

uniform sampler2D color_texture;
uniform sampler2DShadow shadowMap;
uniform sampler2D normal_texture;
uniform sampler2D specular_texture;

uniform vec3 mainLight;
uniform int pLightCount;
uniform vec3 pLightPos[12];

uniform int illuminatedFlag;
uniform vec3 fog_color;

uniform mat4 view;
uniform mat4 DepthBiasMVP;
uniform int myTextureSampler;
uniform bool disableNormalMapping;
uniform bool disableSpecularMapping;

float getFogFactor(){
	const float LOG2 = 1.442695;
	const float density = 1.5;

	float z = gl_FragCoord.z / gl_FragCoord.w /200;
	float depthFogFactor = exp2( -density * density * z * z * LOG2 );
	float groundProximityFogFactor = clamp(1-P.y/50, 0.0, 1.0) *2;
	return clamp(groundProximityFogFactor * clamp(1-depthFogFactor, 0.0, 1.0), 0.0, 1.0 );
}

float getCosTheta(vec3 currentLightDirTangentSpace){
	vec3 TextureNormal_tangentspace = normalize(texture(normal_texture, T).rgb * 2.0 - 1.0);
	vec3 l = normalize(MainLightDirection_tangentspace);
	if (disableNormalMapping){
		return clamp( dot( N,normalize(mainLight-P) ), 0,1 );
	}else{
		return clamp( dot( TextureNormal_tangentspace, l ), 0,1 );
	}
}

float getCosAlpha(vec3 currentLightDirTangentSpace){
	if (disableNormalMapping){
		vec3 view_forward_worldspace = vec3(view[2][0], view[2][1], view[2][2]);
		vec3 E = normalize(view_forward_worldspace);
		vec3 R = reflect(-normalize(mainLight),N);
		return clamp( dot( E,R ), 0,1 );
	}else{
		vec3 TextureNormal_tangentspace = normalize(texture(normal_texture, T).rgb * 2.0 - 1.0);
		vec3 E = normalize(EyeDirection_tangentspace);
		vec3 R = reflect(-normalize(currentLightDirTangentSpace),TextureNormal_tangentspace);
		return clamp( dot( E,R ), 0,1 );
	}
}

void main()
{
	//MaterialConstants
	float specularMapValue = texture(specular_texture, T).x;
	float ka = 0.1; //ambient constant
	float kd = 0.2; //difuse constant
	float ks = disableSpecularMapping ? 0.4 : specularMapValue*2; //specular constant (may depend on specular map)
	float shininess = disableSpecularMapping ? 30 : specularMapValue * 255 /2.0f;

	vec3 Cd = texture(color_texture, T).rgb; //diffuseColor
	float visibility = texture( shadowMap, vec3(S.xy/S.w, (S.z)/S.w) );

	if(visibility <= 0.0f)
	{
		visibility = 0.0f;
	}

	vec3 Ia; //Ambient intensity
	vec3 Id; //Diffiuse intensity
	vec3 Is; //Specular intensity

	//Main light with (directioal) shadows, no attenuation, behaves like a point light far away for calc
	Ia = ka * Cd;
	Id = kd * Cd * visibility * getCosTheta(MainLightDirection_tangentspace);
	Is = ks * visibility * vec3(pow(getCosAlpha(MainLightDirection_tangentspace),shininess));

	//Point lights, no shadows, attenuation
	for (int i = 0; i<pLightCount; i=i+1){
		float LP_distance = sqrt((P.x-pLightPos[i].x)*(P.x-pLightPos[i].x) 
									+(P.y-pLightPos[i].y)*(P.y-pLightPos[i].y) 
									+(P.z-pLightPos[i].z)*(P.z-pLightPos[i].z));
		float attenuation = 1.0f;
		if (LP_distance != 0.0f)
		{
			attenuation = 20/LP_distance;
		}

		Id += kd * Cd * pow(attenuation,2) * getCosTheta(pLightDirection_tangentspace[i]);
		Is += ks * 0.6 * pow(attenuation,2) * vec3(pow(getCosAlpha(pLightDirection_tangentspace[i]),shininess));
	}

	vec3 I = Ia + Id + Is;

	float fogFactor = getFogFactor();
	fragColor= vec4(fogFactor * fog_color + (1-fogFactor)*I,1);
}